@@ -1,10 +1,7 @@
 import sys
 
-from sqlalchemy import Column, ForeignKey, Integer, String, create_engine
-from sqlalchemy.orm import relationship
+from sqlalchemy import Column, Integer, String, create_engine
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.schema import Sequence
-import sqlalchemy.dialects.sqlite
 
 Base = declarative_base()
 
